from zope.component import getUtility

from StringIO import StringIO
from Products.Five.browser import BrowserView

from collective.wtf.interfaces import ICSVWorkflowSerializer
from collective.wtf.exportimport import CSVWorkflowDefinitionConfigurator
        
class ToXML(BrowserView):
    """Export the context workflow to XML as a one-off
    """
    
    def __call__(self):
        
        wfdc = CSVWorkflowDefinitionConfigurator(self.context)
        xml = wfdc.__of__(self.context).generateWorkflowXML()

        self.request.response.setHeader("Content-type","text/xml")
        return xml



from zope.testing import doctest
from unittest import TestSuite

from Testing.ZopeTestCase import FunctionalDocFileSuite
from Products.PloneTestCase.PloneTestCase import PloneTestCase
from Products.PloneTestCase.PloneTestCase import setupPloneSite

from plone.app.controlpanel.tests.cptc import ControlPanelTestCase

setupPloneSite()

OPTIONFLAGS = (doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE)

def test_suite():
    tests = ['workflowed.txt',
             ]
    suite = TestSuite()
    for test in tests:
        suite.addTest(FunctionalDocFileSuite(test,
            optionflags=OPTIONFLAGS,
            package="collective.workflowed.tests",
            test_class=ControlPanelTestCase))
    return suite



## Script (Python) "updateTransition"
##title=Edit content
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=new_state_id
##
title=context.title
description=context.description
trigger_type=context.trigger_type
script_name=context.script_name
after_script_name=context.after_script_name
actbox_name=context.actbox_name
actbox_url=context.actbox_url
actbox_category=context.actbox_category
context.setProperties(title=title,description=description,trigger_type=trigger_type,script_name=script_name,after_script_name=after_script_name,new_state_id=new_state_id,actbox_name=actbox_name,actbox_url=actbox_url,actbox_category=actbox_category)



from zope import schema
from zope.interface import Interface
from zope.component import adapts
from zope.interface import implements
from zope.formlib import form

from Acquisition import aq_inner

from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.CMFCore.utils import getToolByName
from Products.Five.formlib import formbase
from Products.statusmessages.interfaces import IStatusMessage
from Products.CMFDefault.formlib.schema import SchemaAdapterBase

from plone.app.form.validators import null_validator
from plone.app.controlpanel.form import ControlPanelForm

from collective.workflowed import WorkflowedMessageFactory as _

class IAddWorkflowForm(Interface):
    """Define the fields for the new workflow
    """
    title=schema.TextLine(title=_(u"Title"),
                          description=_(u"This title will be shown in all workflow related control panels."),
                          required=True)
    description=schema.Text(title=_(u"Description"),
                            description=_(u"Free text, but some UI methods expect a bullet point list of characteristics in REST format."),
                            required=False)
    based_on=schema.Choice(title=_(u"Based on"),
                           description=_(u"Which workflow to use as a base. For empty workflows choose the single state workflow."),
                           missing_value=tuple(),
                           vocabulary="plone.app.vocabularies.Workflows",
                           required=True)

class AddWorkflowFormAdapter(SchemaAdapterBase):
    adapts(IPloneSiteRoot)
    implements(IAddWorkflowForm)
    title=''
    description=''
    based_on='one_state_workflow'

class AddWorkflowForm(ControlPanelForm):
    form_fields = form.FormFields(IAddWorkflowForm)
    form_name = _(u"Add workflow")
    label = _(u"Add workflow")
    description = _(u"Please enter the title and description for the new workflow and choose on which existing workflow it will be based.")
    
    def __call__(self):
        self.request.set('disable_border',True)
        return super(AddWorkflowForm,self).__call__()

    @form.action(_(u"Add"))
    def action_add(self, action, data):
        """Add a new workflow to the workflow tool
        """
        IStatusMessage(self.request).addStatusMessage(_(u"Workflow added."),type="info")
        context = aq_inner(self.context)
        wf_tool = getToolByName(context,'portal_workflow')
        plone_utils = getToolByName(self.context, 'plone_utils')
        workflow_id=plone_utils.normalizeString(data['title'])
        wf_tool.manage_pasteObjects(wf_tool.manage_copyObjects([data['based_on']]))
        wf_tool.manage_renameObject('copy_of_%s' % data['based_on'], workflow_id)
        wf_tool[workflow_id].setProperties(title=data['title'],description=data['description'])
        self.context.request.response.redirect('%s/@@workflowed-controlpanel?selected_workflow=%s' % (context.absolute_url(),workflow_id))
        return ''

    @form.action(_(u'label_cancel', default=u'Cancel'),
                 validator=null_validator,
                 name=u'cancel')
    def action_cancel(self, action, data):
        IStatusMessage(self.request).addStatusMessage(_("Add new workflow action canceled."),
                                                      type="info")
        self.request.response.redirect('%s/@@workflowed-controlpanel' % self.context.absolute_url())
        return ''




from plone.app.workflow.remap import remap_workflow
from plone.memoize.instance import memoize

from zope.component import getUtility
from zope.i18n import translate
from zope.schema.interfaces import IVocabularyFactory

from Acquisition import aq_inner

from Products.CMFCore.utils import getToolByName
from Products.CMFEditions.setuphandlers import DEFAULT_POLICIES
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone import PloneMessageFactory as pmf
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.app.controlpanel.form import ControlPanelView



class WorkflowedControlPanel(ControlPanelView):

    def __init__(self,context,request):
        super(WorkflowedControlPanel, self).__init__(context, request)
        request.set('disable_border', True)
        form = request.form
        submitted = form.get('form.submitted', False)
        save_button = form.get('form.button.Save', None) is not None
        cancel_button = form.get('form.button.Cancel', None) is not None
        if submitted and not cancel_button:
            request.response.redirect('%s/@@workflowed-controlpanel?selected_workflow=%s' % (context.absolute_url() , self.selected_workflow()))
        elif cancel_button:
            request.response.redirect(context.absolute_url() + '/plone_control_panel')

    @memoize
    def selected_workflow(self):
        context = aq_inner(self.context)
        form = self.request.form
        current_workflow=form.get('selected_workflow',self.available_workflows()[0]['id'])
        return current_workflow

    @memoize
    def available_workflows(self):
        vocab_factory = getUtility(IVocabularyFactory,
                                   name="plone.app.vocabularies.Workflows")
        workflows = []
        for v in vocab_factory(self.context):
            if v.title:
                title = translate(v.title, context=self.request)
            else:
                title = translate(v.token, domain='plone', context=self.request)
            workflows.append(dict(id=v.value, title=title) )
        def _key(v):
            return v['title']
        workflows.sort(key=_key)
        return workflows

    @memoize
    def current_workflow_url(self):
        context = aq_inner(self.context)
        portal_workflow = getToolByName(context, 'portal_workflow')
        wf = getattr(portal_workflow, self.selected_workflow())
        return wf.absolute_url()

    @memoize
    def current_workflow_data(self):
        context = aq_inner(self.context)
        portal_workflow = getToolByName(context, 'portal_workflow')
        wf = getattr(portal_workflow, self.selected_workflow())
        text = translate(wf.description.strip(), domain='plone', context=self.request)
        return dict(title=wf.title,description=[s.strip() for s in text.split('- ') if s],initial_state=wf.states.initial_state,transitions=wf.transitions.objectValues(),states=wf.states.objectValues())

    def generate_workflow_js(self):
        wf=self.current_workflow_data()
        javascript= """var workflow=new Workflow('workflow_editor');
        var w = new GUIPalette();
        workflow.setToolWindow(w);
        w.setPosition(20,20);
        var dialog = new VectorPropertyWindow();
        workflow.addFigure(dialog,700,20);
        """
        state_template="""
          var figure = new StateWindow("%s");
          figure.setDimension(160,40);
          figure_outputPort = new OutputPort();
          figure_outputPort.setDirection(-1);
          figure_outputPort.setMaxFanOut(1000);
          figure_outputPort.setWorkflow(workflow);
          figure_outputPort.setBackgroundColor(new Color(245,115,115));
          figure_inputPort = new InputPort();
          figure_inputPort.setDirection(-1);
          figure_inputPort.setWorkflow(workflow);
          figure_inputPort.setBackgroundColor(new Color(115,245,115));
          figure_inputPort.setColor(null);
          figure.addPort(figure_outputPort,160,25);
          figure.addPort(figure_inputPort,160,40);
          workflow.addFigure(figure,%d,%d);
        """
        x=150
        y=20
        for state in wf['states']:
            new_state=state_template % (state.id,x,y)
            javascript=javascript + new_state.replace('figure','st_%s' % state.id)
            if wf['initial_state']==state.id:
                javascript=javascript+"st_%s.setLineWidth(3)\n" % state.id
            y=y+120
        transition_template="""
          var figure = new TransitionWindow("%s");
          figure.setDimension(120,40);
          figure_outputPort = new OutputPort();
          figure_outputPort.setDirection(-1); 
          figure_outputPort.setMaxFanOut(1); 
          figure_outputPort.setWorkflow(workflow);
          figure_outputPort.setBackgroundColor(new Color(115,245,115));
          figure_inputPort = new InputPort();
          figure_inputPort.setDirection(-1);
          figure_inputPort.setWorkflow(workflow);
          figure_inputPort.setBackgroundColor(new Color(245,115,111));
          figure_inputPort.setColor(null);
          figure.addPort(figure_outputPort,0,25);
          figure.addPort(figure_inputPort,0,40);
          workflow.addFigure(figure,%d,%d);
        """
        x=500
        y=20
        for transition in wf['transitions']:
            new_transition=transition_template % (transition.id,x,y)
            javascript=javascript + new_transition.replace('figure','tr_%s' % transition.id)
            y=y+90
        for state in wf['states']:
            for transition in state.transitions:
                javascript=javascript+"""
          if (typeof(tr_%s_inputPort)!="undefined") {
            var connection = new Connection();
            connection.setSource(st_%s_outputPort);
            connection.setTarget(tr_%s_inputPort);
            workflow.addFigure(connection);
          }
        """ % (transition,state.id,transition)
        for transition in wf['transitions']:
                javascript=javascript+"""
          if (typeof(st_%s_inputPort)!="undefined") {
            var connection = new Connection();
            connection.setSource(tr_%s_outputPort);
            connection.setTarget(st_%s_inputPort);
            workflow.addFigure(connection);
          }
        """ % (transition.new_state_id,transition.id,transition.new_state_id)
        return javascript



## Script (Python) "updateState"
##title=Edit content
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=transitions=[]
##
title=context.title
description=context.description
context.setProperties(title=title,transitions=transitions,description=description)



from setuptools import setup, find_packages

version = '1.0'

setup(name='collective.workflowed',
      version=version,
      description="A graphical workflow editor for Plone",
      long_description="""\
A javascript based graphical workflow editor for Plone.""",
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='zope plone workflow editor',
      author='Carlos de la Guardia',
      author_email='cguardia@yahoo.com',
      url='http://svn.plone.org/svn/collective/collective.workflowed',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'collective.wtf',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
